OutputQueueableProvider

Trait OutputQueueableProvider 

pub trait OutputQueueableProvider<'a, B: BufferHandles> {
    type Queueable: OutputQueueable<B>;
}
Trait for all objects that are capable of providing objects that can be queued to the CAPTURE queue.

Required Associated Types

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl<'a, B, Q> OutputQueueableProvider<'a, B> for Q
where B: BufferHandles, Q: QueueableProvider<'a>, Q::Queueable: OutputQueueable<B>,

Source

type Queueable = <Q as QueueableProvider<'a>>::Queueable

Source

impl<'a, OP, P, InputDoneCb, DecoderEventCb, FormatChangedCb> OutputQueueableProvider<'a, OP> for Decoder<Decoding<OP, P, InputDoneCb, DecoderEventCb, FormatChangedCb>>
where Queue<Output, BuffersAllocated<OP>>: OutputQueueableProvider<'a, OP>, OP: BufferHandles, P: HandlesProvider, InputDoneCb: InputDoneCallback<OP>, DecoderEventCb: DecoderEventCallback<P>, FormatChangedCb: FormatChangedCallback<P>,

Source

impl<'a, OP, P, InputDoneCb, OutputReadyCb> OutputQueueableProvider<'a, OP> for Encoder<Encoding<OP, P, InputDoneCb, OutputReadyCb>>