pub trait OutputQueueableProvider<'a, B: BufferHandles> {
type Queueable: OutputQueueable<B>;
}Expand description
Trait for all objects that are capable of providing objects that can be queued to the CAPTURE queue.
Required Associated Types
type Queueable: OutputQueueable<B>
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".